/*
 * @OSF_COPYRIGHT@
 * COPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993, 1996 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (DCE).  See the file named COPYRIGHT.DCE in the
 * src directory for the full copyright text.
 */
/*
 * HISTORY
 * $Log: cksum.c,v $
 * Revision 1.1.8.2  1996/02/17  23:59:33  marty
 * 	Update OSF copyright years
 * 	[1996/02/17  22:53:13  marty]
 *
 * Revision 1.1.8.1  1995/12/08  00:13:26  root
 * 	Submit OSF/DCE 1.2.1
 * 	[1995/12/07  23:55:20  root]
 * 
 * Revision 1.1.6.1  1994/01/21  22:30:45  cbrooks
 * 	Code Cleanup
 * 	[1994/01/21  20:20:49  cbrooks]
 * 
 * Revision 1.1.4.3  1993/01/03  22:34:30  bbelch
 * 	Embedding copyright notice
 * 	[1993/01/03  19:49:31  bbelch]
 * 
 * Revision 1.1.4.2  1992/12/23  19:35:09  zeliff
 * 	Embedding copyright notice
 * 	[1992/12/23  01:15:08  zeliff]
 * 
 * Revision 1.1.2.2  1992/05/01  17:36:05  rsalz
 * 	"Changed pas part of rpc6 drop."
 * 	[1992/05/01  17:33:47  rsalz]
 * 
 * Revision 1.1  1992/01/19  03:16:49  devrcs
 * 	Initial revision
 * 
 * $EndLog$
 */
/*
*/

/*  cksum.c V=1 11/07/90 //crown/prgy/krb5/lib/destoo
**
** Copyright (c) Hewlett-Packard Company 1991
** Unpublished work. All Rights Reserved.
**
**    ** OSF DCE Confidential - DCE Security Component BL 6   **
**    ** OSF DCE AUTH 3 - prerelease                          **
**
*/
/*
 * Copyright (c) 1990 Dennis Ferguson.  All rights reserved.
 *
 * Commercial use is permitted only if products which are derived from
 * or include this software are made available for purchase and/or use
 * in Canada.  Otherwise, redistribution and use in source and binary
 * forms are permitted.
 */

/*
 * des_cbc_cksum.c - compute an 8 byte checksum using DES in CBC mode
 */
#include <dce/dce.h>
#include <des.h>
#include <des_tables.h>

unsigned long
des_cbc_cksum
#ifdef _DCE_PROTO_
    (
	des_cblock *in,
	des_cblock *out,
	long length,
	des_key_schedule schedule,
	des_cblock ivec
     )
#else 
(in, out, length, schedule, ivec)
	des_cblock *in;
	des_cblock *out;
	long length;
	des_key_schedule schedule;
	des_cblock ivec;
#endif
{
	register unsigned long left, right;
	register unsigned long temp;
	register unsigned long *kp;
	register unsigned char *ip;
	register long len;

	/*
	 * Initialize left and right with the contents of the initial
	 * vector.
	 */
	ip = (unsigned char *)ivec;
	GET_HALF_BLOCK(left, ip);
	GET_HALF_BLOCK(right, ip);

	/*
	 * Suitably initialized, now work the length down 8 bytes
	 * at a time.
	 */
	ip = (unsigned char *)in;
	len = length;
	while (len > 0) {
		/*
		 * Get more input, xor it in.  If the length is
		 * greater than or equal to 8 this is straight
		 * forward.  Otherwise we have to fart around.
		 */
		if (len >= 8) {
			left ^= ((*ip++) & 0xff) << 24;
			left ^= ((*ip++) & 0xff) << 16;
			left ^= ((*ip++) & 0xff) << 8;
			left ^= (*ip++) & 0xff;
			right ^= ((*ip++) & 0xff) << 24;
			right ^= ((*ip++) & 0xff) << 16;
			right ^= ((*ip++) & 0xff) << 8;
			right ^= (*ip++) & 0xff;
			len -= 8;
		} else {
			/*
			 * Oh, shoot.  We need to pad the
			 * end with zeroes.  Work backwards
			 * to do this.
			 */
			ip += (int) len;
			switch((int)len) {
			case 7:
				right ^= (*(--ip) & 0xff) << 8;
			case 6:
				right ^= (*(--ip) & 0xff) << 16;
			case 5:
				right ^= (*(--ip) & 0xff) << 24;
			case 4:
				left ^= *(--ip) & 0xff;
			case 3:
				left ^= (*(--ip) & 0xff) << 8;
			case 2:
				left ^= (*(--ip) & 0xff) << 16;
			case 1:
				left ^= (*(--ip) & 0xff) << 24;
				break;
			}
			len = 0;
		}

		/*
		 * Encrypt what we have
		 */
		kp = (unsigned long *)schedule;
		DES_DO_ENCRYPT(left, right, temp, kp);
	}

	/*
	 * Done.  Left and right have the checksum.  Put it into
	 * the output.
	 */
	ip = (unsigned char *)out;
	PUT_HALF_BLOCK(left, ip);
	PUT_HALF_BLOCK(right, ip);

	/*
	 * Return right.  I'll bet the MIT code returns this
	 * inconsistantly (with the low order byte of the checksum
	 * not always in the low order byte of the long).  We won't.
	 */
	return right;
}
