/*
 * @OSF_COPYRIGHT@
 * COPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993, 1996 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (DCE).  See the file named COPYRIGHT.DCE in the
 * src directory for the full copyright text.
 */
/*
 * HISTORY
 * $Log: des_tables.c,v $
 * Revision 1.1.8.2  1996/02/17  23:59:38  marty
 * 	Update OSF copyright years
 * 	[1996/02/17  22:53:14  marty]
 *
 * Revision 1.1.8.1  1995/12/08  00:13:30  root
 * 	Submit OSF/DCE 1.2.1
 * 	[1995/12/07  23:55:22  root]
 * 
 * Revision 1.1.6.1  1994/01/21  22:30:49  cbrooks
 * 	Code Cleanup
 * 	[1994/01/21  20:20:52  cbrooks]
 * 
 * Revision 1.1.4.3  1993/01/03  22:34:36  bbelch
 * 	Embedding copyright notice
 * 	[1993/01/03  19:49:41  bbelch]
 * 
 * Revision 1.1.4.2  1992/12/23  19:35:31  zeliff
 * 	Embedding copyright notice
 * 	[1992/12/23  01:15:19  zeliff]
 * 
 * Revision 1.1.2.2  1992/05/01  17:36:21  rsalz
 * 	"Changed pas part of rpc6 drop."
 * 	[1992/05/01  17:34:00  rsalz]
 * 
 * Revision 1.1  1992/01/19  03:16:52  devrcs
 * 	Initial revision
 * 
 * $EndLog$
 */
/*
*/

/*  des_tables.c V=1 11/07/90 //crown/prgy/krb5/lib/destoo
**
** Copyright (c) Hewlett-Packard Company 1991
** Unpublished work. All Rights Reserved.
**
**    ** OSF DCE Confidential - DCE Security Component BL 6   **
**    ** OSF DCE AUTH 3 - prerelease                          **
**
*/
/*
 * Copyright (c) 1990 Dennis Ferguson.  All rights reserved.
 *
 * Commercial use is permitted only if products which are derived from
 * or include this software are made available for purchase and/or use
 * in Canada.  Otherwise, redistribution and use in source and binary
 * forms are permitted.
 */

/*
 * des_tables.c - precomputed tables used for the DES cipher function
 */

/*
 * Include the header file so something will complain if the
 * declarations get out of sync
 */
#include <dce/dce.h>
#include <des_tables.h>

/*
 * These tables may be declared const if you want.  Many compilers
 * don't support this, though.
 */

/*
 * The DES algorithm which uses these is intended to be fairly speedy
 * at the expense of some memory.  All the standard hacks are used.
 * The S boxes and the P permutation are precomputed into one table.
 * The E box never actually appears explicitly since it is easy to apply
 * this algorithmically as needed.  The initial permutation and final
 * (inverse initial) permutation are computed from tables designed to
 * permute one byte at a time.  This should run pretty fast on machines
 * with 32 bit words and bit field/multiple bit shift instructions which
 * are fast.
 */

/*
 * The initial permutation array.  This is used to compute both the
 * left and the right halves of the initial permutation using bytes
 * from words made from the following operations:
 *
 * ((left & 0x55555555) << 1) | (right & 0x55555555)  for left half
 * (left & 0xaaaaaaaa) | ((right & 0xaaaaaaaa) >> 1)  for right half
 *
 * The scheme is that we index into the table using each byte.  The
 * result from the high order byte is or'd with the result from the
 * next byte shifted left once is or'd with the result from the next
 * byte shifted left twice if or'd with the result from the low order
 * byte shifted left by three.  Clear?
 */

unsigned long des_IP_table[256] = {
	0x00000000UL, 0x00000010UL, 0x00000001UL, 0x00000011UL,
	0x00001000UL, 0x00001010UL, 0x00001001UL, 0x00001011UL,
	0x00000100UL, 0x00000110UL, 0x00000101UL, 0x00000111UL,
	0x00001100UL, 0x00001110UL, 0x00001101UL, 0x00001111UL,
	0x00100000UL, 0x00100010UL, 0x00100001UL, 0x00100011UL,
	0x00101000UL, 0x00101010UL, 0x00101001UL, 0x00101011UL,
	0x00100100UL, 0x00100110UL, 0x00100101UL, 0x00100111UL,
	0x00101100UL, 0x00101110UL, 0x00101101UL, 0x00101111UL,
	0x00010000UL, 0x00010010UL, 0x00010001UL, 0x00010011UL,
	0x00011000UL, 0x00011010UL, 0x00011001UL, 0x00011011UL,
	0x00010100UL, 0x00010110UL, 0x00010101UL, 0x00010111UL,
	0x00011100UL, 0x00011110UL, 0x00011101UL, 0x00011111UL,
	0x00110000UL, 0x00110010UL, 0x00110001UL, 0x00110011UL,
	0x00111000UL, 0x00111010UL, 0x00111001UL, 0x00111011UL,
	0x00110100UL, 0x00110110UL, 0x00110101UL, 0x00110111UL,
	0x00111100UL, 0x00111110UL, 0x00111101UL, 0x00111111UL,
	0x10000000UL, 0x10000010UL, 0x10000001UL, 0x10000011UL,
	0x10001000UL, 0x10001010UL, 0x10001001UL, 0x10001011UL,
	0x10000100UL, 0x10000110UL, 0x10000101UL, 0x10000111UL,
	0x10001100UL, 0x10001110UL, 0x10001101UL, 0x10001111UL,
	0x10100000UL, 0x10100010UL, 0x10100001UL, 0x10100011UL,
	0x10101000UL, 0x10101010UL, 0x10101001UL, 0x10101011UL,
	0x10100100UL, 0x10100110UL, 0x10100101UL, 0x10100111UL,
	0x10101100UL, 0x10101110UL, 0x10101101UL, 0x10101111UL,
	0x10010000UL, 0x10010010UL, 0x10010001UL, 0x10010011UL,
	0x10011000UL, 0x10011010UL, 0x10011001UL, 0x10011011UL,
	0x10010100UL, 0x10010110UL, 0x10010101UL, 0x10010111UL,
	0x10011100UL, 0x10011110UL, 0x10011101UL, 0x10011111UL,
	0x10110000UL, 0x10110010UL, 0x10110001UL, 0x10110011UL,
	0x10111000UL, 0x10111010UL, 0x10111001UL, 0x10111011UL,
	0x10110100UL, 0x10110110UL, 0x10110101UL, 0x10110111UL,
	0x10111100UL, 0x10111110UL, 0x10111101UL, 0x10111111UL,
	0x01000000UL, 0x01000010UL, 0x01000001UL, 0x01000011UL,
	0x01001000UL, 0x01001010UL, 0x01001001UL, 0x01001011UL,
	0x01000100UL, 0x01000110UL, 0x01000101UL, 0x01000111UL,
	0x01001100UL, 0x01001110UL, 0x01001101UL, 0x01001111UL,
	0x01100000UL, 0x01100010UL, 0x01100001UL, 0x01100011UL,
	0x01101000UL, 0x01101010UL, 0x01101001UL, 0x01101011UL,
	0x01100100UL, 0x01100110UL, 0x01100101UL, 0x01100111UL,
	0x01101100UL, 0x01101110UL, 0x01101101UL, 0x01101111UL,
	0x01010000UL, 0x01010010UL, 0x01010001UL, 0x01010011UL,
	0x01011000UL, 0x01011010UL, 0x01011001UL, 0x01011011UL,
	0x01010100UL, 0x01010110UL, 0x01010101UL, 0x01010111UL,
	0x01011100UL, 0x01011110UL, 0x01011101UL, 0x01011111UL,
	0x01110000UL, 0x01110010UL, 0x01110001UL, 0x01110011UL,
	0x01111000UL, 0x01111010UL, 0x01111001UL, 0x01111011UL,
	0x01110100UL, 0x01110110UL, 0x01110101UL, 0x01110111UL,
	0x01111100UL, 0x01111110UL, 0x01111101UL, 0x01111111UL,
	0x11000000UL, 0x11000010UL, 0x11000001UL, 0x11000011UL,
	0x11001000UL, 0x11001010UL, 0x11001001UL, 0x11001011UL,
	0x11000100UL, 0x11000110UL, 0x11000101UL, 0x11000111UL,
	0x11001100UL, 0x11001110UL, 0x11001101UL, 0x11001111UL,
	0x11100000UL, 0x11100010UL, 0x11100001UL, 0x11100011UL,
	0x11101000UL, 0x11101010UL, 0x11101001UL, 0x11101011UL,
	0x11100100UL, 0x11100110UL, 0x11100101UL, 0x11100111UL,
	0x11101100UL, 0x11101110UL, 0x11101101UL, 0x11101111UL,
	0x11010000UL, 0x11010010UL, 0x11010001UL, 0x11010011UL,
	0x11011000UL, 0x11011010UL, 0x11011001UL, 0x11011011UL,
	0x11010100UL, 0x11010110UL, 0x11010101UL, 0x11010111UL,
	0x11011100UL, 0x11011110UL, 0x11011101UL, 0x11011111UL,
	0x11110000UL, 0x11110010UL, 0x11110001UL, 0x11110011UL,
	0x11111000UL, 0x11111010UL, 0x11111001UL, 0x11111011UL,
	0x11110100UL, 0x11110110UL, 0x11110101UL, 0x11110111UL,
	0x11111100UL, 0x11111110UL, 0x11111101UL, 0x11111111UL
};

/*
 * The final permutation array.  Like the IP array, used
 * to compute both the left and right results from the bytes
 * of words computed from:
 *
 * ((left & 0x0f0f0f0f) << 4) | (right & 0x0f0f0f0f)  for left result
 * (left & 0xf0f0f0f0) | ((right & 0xf0f0f0f0) >> 4)  for right result
 *
 * The result from the high order byte is shifted left 6 bits and
 * or'd with the result from the next byte shifted left 4 bits, which
 * is or'd with the result from the next byte shifted left 2 bits,
 * which is or'd with the result from the low byte.
 */
unsigned long des_FP_table[256] = {
	0x00000000UL, 0x02000000UL, 0x00020000UL, 0x02020000UL,
	0x00000200UL, 0x02000200UL, 0x00020200UL, 0x02020200UL,
	0x00000002UL, 0x02000002UL, 0x00020002UL, 0x02020002UL,
	0x00000202UL, 0x02000202UL, 0x00020202UL, 0x02020202UL,
	0x01000000UL, 0x03000000UL, 0x01020000UL, 0x03020000UL,
	0x01000200UL, 0x03000200UL, 0x01020200UL, 0x03020200UL,
	0x01000002UL, 0x03000002UL, 0x01020002UL, 0x03020002UL,
	0x01000202UL, 0x03000202UL, 0x01020202UL, 0x03020202UL,
	0x00010000UL, 0x02010000UL, 0x00030000UL, 0x02030000UL,
	0x00010200UL, 0x02010200UL, 0x00030200UL, 0x02030200UL,
	0x00010002UL, 0x02010002UL, 0x00030002UL, 0x02030002UL,
	0x00010202UL, 0x02010202UL, 0x00030202UL, 0x02030202UL,
	0x01010000UL, 0x03010000UL, 0x01030000UL, 0x03030000UL,
	0x01010200UL, 0x03010200UL, 0x01030200UL, 0x03030200UL,
	0x01010002UL, 0x03010002UL, 0x01030002UL, 0x03030002UL,
	0x01010202UL, 0x03010202UL, 0x01030202UL, 0x03030202UL,
	0x00000100UL, 0x02000100UL, 0x00020100UL, 0x02020100UL,
	0x00000300UL, 0x02000300UL, 0x00020300UL, 0x02020300UL,
	0x00000102UL, 0x02000102UL, 0x00020102UL, 0x02020102UL,
	0x00000302UL, 0x02000302UL, 0x00020302UL, 0x02020302UL,
	0x01000100UL, 0x03000100UL, 0x01020100UL, 0x03020100UL,
	0x01000300UL, 0x03000300UL, 0x01020300UL, 0x03020300UL,
	0x01000102UL, 0x03000102UL, 0x01020102UL, 0x03020102UL,
	0x01000302UL, 0x03000302UL, 0x01020302UL, 0x03020302UL,
	0x00010100UL, 0x02010100UL, 0x00030100UL, 0x02030100UL,
	0x00010300UL, 0x02010300UL, 0x00030300UL, 0x02030300UL,
	0x00010102UL, 0x02010102UL, 0x00030102UL, 0x02030102UL,
	0x00010302UL, 0x02010302UL, 0x00030302UL, 0x02030302UL,
	0x01010100UL, 0x03010100UL, 0x01030100UL, 0x03030100UL,
	0x01010300UL, 0x03010300UL, 0x01030300UL, 0x03030300UL,
	0x01010102UL, 0x03010102UL, 0x01030102UL, 0x03030102UL,
	0x01010302UL, 0x03010302UL, 0x01030302UL, 0x03030302UL,
	0x00000001UL, 0x02000001UL, 0x00020001UL, 0x02020001UL,
	0x00000201UL, 0x02000201UL, 0x00020201UL, 0x02020201UL,
	0x00000003UL, 0x02000003UL, 0x00020003UL, 0x02020003UL,
	0x00000203UL, 0x02000203UL, 0x00020203UL, 0x02020203UL,
	0x01000001UL, 0x03000001UL, 0x01020001UL, 0x03020001UL,
	0x01000201UL, 0x03000201UL, 0x01020201UL, 0x03020201UL,
	0x01000003UL, 0x03000003UL, 0x01020003UL, 0x03020003UL,
	0x01000203UL, 0x03000203UL, 0x01020203UL, 0x03020203UL,
	0x00010001UL, 0x02010001UL, 0x00030001UL, 0x02030001UL,
	0x00010201UL, 0x02010201UL, 0x00030201UL, 0x02030201UL,
	0x00010003UL, 0x02010003UL, 0x00030003UL, 0x02030003UL,
	0x00010203UL, 0x02010203UL, 0x00030203UL, 0x02030203UL,
	0x01010001UL, 0x03010001UL, 0x01030001UL, 0x03030001UL,
	0x01010201UL, 0x03010201UL, 0x01030201UL, 0x03030201UL,
	0x01010003UL, 0x03010003UL, 0x01030003UL, 0x03030003UL,
	0x01010203UL, 0x03010203UL, 0x01030203UL, 0x03030203UL,
	0x00000101UL, 0x02000101UL, 0x00020101UL, 0x02020101UL,
	0x00000301UL, 0x02000301UL, 0x00020301UL, 0x02020301UL,
	0x00000103UL, 0x02000103UL, 0x00020103UL, 0x02020103UL,
	0x00000303UL, 0x02000303UL, 0x00020303UL, 0x02020303UL,
	0x01000101UL, 0x03000101UL, 0x01020101UL, 0x03020101UL,
	0x01000301UL, 0x03000301UL, 0x01020301UL, 0x03020301UL,
	0x01000103UL, 0x03000103UL, 0x01020103UL, 0x03020103UL,
	0x01000303UL, 0x03000303UL, 0x01020303UL, 0x03020303UL,
	0x00010101UL, 0x02010101UL, 0x00030101UL, 0x02030101UL,
	0x00010301UL, 0x02010301UL, 0x00030301UL, 0x02030301UL,
	0x00010103UL, 0x02010103UL, 0x00030103UL, 0x02030103UL,
	0x00010303UL, 0x02010303UL, 0x00030303UL, 0x02030303UL,
	0x01010101UL, 0x03010101UL, 0x01030101UL, 0x03030101UL,
	0x01010301UL, 0x03010301UL, 0x01030301UL, 0x03030301UL,
	0x01010103UL, 0x03010103UL, 0x01030103UL, 0x03030103UL,
	0x01010303UL, 0x03010303UL, 0x01030303UL, 0x03030303UL
};


/*
 * The SP table is actually the S boxes and the P permutation
 * table combined.  This table is actually reordered from the
 * spec, to match the order of key application we follow.
 */
unsigned long des_SP_table[8][64] = {
	0x00100000UL, 0x02100001UL, 0x02000401UL, 0x00000000UL,	/* 7 */
	0x00000400UL, 0x02000401UL, 0x00100401UL, 0x02100400UL,
	0x02100401UL, 0x00100000UL, 0x00000000UL, 0x02000001UL,
	0x00000001UL, 0x02000000UL, 0x02100001UL, 0x00000401UL,
	0x02000400UL, 0x00100401UL, 0x00100001UL, 0x02000400UL,
	0x02000001UL, 0x02100000UL, 0x02100400UL, 0x00100001UL,
	0x02100000UL, 0x00000400UL, 0x00000401UL, 0x02100401UL,
	0x00100400UL, 0x00000001UL, 0x02000000UL, 0x00100400UL,
	0x02000000UL, 0x00100400UL, 0x00100000UL, 0x02000401UL,
	0x02000401UL, 0x02100001UL, 0x02100001UL, 0x00000001UL,
	0x00100001UL, 0x02000000UL, 0x02000400UL, 0x00100000UL,
	0x02100400UL, 0x00000401UL, 0x00100401UL, 0x02100400UL,
	0x00000401UL, 0x02000001UL, 0x02100401UL, 0x02100000UL,
	0x00100400UL, 0x00000000UL, 0x00000001UL, 0x02100401UL,
	0x00000000UL, 0x00100401UL, 0x02100000UL, 0x00000400UL,
	0x02000001UL, 0x02000400UL, 0x00000400UL, 0x00100001UL,

	0x00808200UL, 0x00000000UL, 0x00008000UL, 0x00808202UL,	/* 1 */
	0x00808002UL, 0x00008202UL, 0x00000002UL, 0x00008000UL,
	0x00000200UL, 0x00808200UL, 0x00808202UL, 0x00000200UL,
	0x00800202UL, 0x00808002UL, 0x00800000UL, 0x00000002UL,
	0x00000202UL, 0x00800200UL, 0x00800200UL, 0x00008200UL,
	0x00008200UL, 0x00808000UL, 0x00808000UL, 0x00800202UL,
	0x00008002UL, 0x00800002UL, 0x00800002UL, 0x00008002UL,
	0x00000000UL, 0x00000202UL, 0x00008202UL, 0x00800000UL,
	0x00008000UL, 0x00808202UL, 0x00000002UL, 0x00808000UL,
	0x00808200UL, 0x00800000UL, 0x00800000UL, 0x00000200UL,
	0x00808002UL, 0x00008000UL, 0x00008200UL, 0x00800002UL,
	0x00000200UL, 0x00000002UL, 0x00800202UL, 0x00008202UL,
	0x00808202UL, 0x00008002UL, 0x00808000UL, 0x00800202UL,
	0x00800002UL, 0x00000202UL, 0x00008202UL, 0x00808200UL,
	0x00000202UL, 0x00800200UL, 0x00800200UL, 0x00000000UL,
	0x00008002UL, 0x00008200UL, 0x00000000UL, 0x00808002UL,

	0x00000104UL, 0x04010100UL, 0x00000000UL, 0x04010004UL,	/* 3 */
	0x04000100UL, 0x00000000UL, 0x00010104UL, 0x04000100UL,
	0x00010004UL, 0x04000004UL, 0x04000004UL, 0x00010000UL,
	0x04010104UL, 0x00010004UL, 0x04010000UL, 0x00000104UL,
	0x04000000UL, 0x00000004UL, 0x04010100UL, 0x00000100UL,
	0x00010100UL, 0x04010000UL, 0x04010004UL, 0x00010104UL,
	0x04000104UL, 0x00010100UL, 0x00010000UL, 0x04000104UL,
	0x00000004UL, 0x04010104UL, 0x00000100UL, 0x04000000UL,
	0x04010100UL, 0x04000000UL, 0x00010004UL, 0x00000104UL,
	0x00010000UL, 0x04010100UL, 0x04000100UL, 0x00000000UL,
	0x00000100UL, 0x00010004UL, 0x04010104UL, 0x04000100UL,
	0x04000004UL, 0x00000100UL, 0x00000000UL, 0x04010004UL,
	0x04000104UL, 0x00010000UL, 0x04000000UL, 0x04010104UL,
	0x00000004UL, 0x00010104UL, 0x00010100UL, 0x04000004UL,
	0x04010000UL, 0x04000104UL, 0x00000104UL, 0x04010000UL,
	0x00010104UL, 0x00000004UL, 0x04010004UL, 0x00010100UL,

	0x00000080UL, 0x01040080UL, 0x01040000UL, 0x21000080UL,	/* 5 */
	0x00040000UL, 0x00000080UL, 0x20000000UL, 0x01040000UL,
	0x20040080UL, 0x00040000UL, 0x01000080UL, 0x20040080UL,
	0x21000080UL, 0x21040000UL, 0x00040080UL, 0x20000000UL,
	0x01000000UL, 0x20040000UL, 0x20040000UL, 0x00000000UL,
	0x20000080UL, 0x21040080UL, 0x21040080UL, 0x01000080UL,
	0x21040000UL, 0x20000080UL, 0x00000000UL, 0x21000000UL,
	0x01040080UL, 0x01000000UL, 0x21000000UL, 0x00040080UL,
	0x00040000UL, 0x21000080UL, 0x00000080UL, 0x01000000UL,
	0x20000000UL, 0x01040000UL, 0x21000080UL, 0x20040080UL,
	0x01000080UL, 0x20000000UL, 0x21040000UL, 0x01040080UL,
	0x20040080UL, 0x00000080UL, 0x01000000UL, 0x21040000UL,
	0x21040080UL, 0x00040080UL, 0x21000000UL, 0x21040080UL,
	0x01040000UL, 0x00000000UL, 0x20040000UL, 0x21000000UL,
	0x00040080UL, 0x01000080UL, 0x20000080UL, 0x00040000UL,
	0x00000000UL, 0x20040000UL, 0x01040080UL, 0x20000080UL,

	0x80401000UL, 0x80001040UL, 0x80001040UL, 0x00000040UL,	/* 4 */
	0x00401040UL, 0x80400040UL, 0x80400000UL, 0x80001000UL,
	0x00000000UL, 0x00401000UL, 0x00401000UL, 0x80401040UL,
	0x80000040UL, 0x00000000UL, 0x00400040UL, 0x80400000UL,
	0x80000000UL, 0x00001000UL, 0x00400000UL, 0x80401000UL,
	0x00000040UL, 0x00400000UL, 0x80001000UL, 0x00001040UL,
	0x80400040UL, 0x80000000UL, 0x00001040UL, 0x00400040UL,
	0x00001000UL, 0x00401040UL, 0x80401040UL, 0x80000040UL,
	0x00400040UL, 0x80400000UL, 0x00401000UL, 0x80401040UL,
	0x80000040UL, 0x00000000UL, 0x00000000UL, 0x00401000UL,
	0x00001040UL, 0x00400040UL, 0x80400040UL, 0x80000000UL,
	0x80401000UL, 0x80001040UL, 0x80001040UL, 0x00000040UL,
	0x80401040UL, 0x80000040UL, 0x80000000UL, 0x00001000UL,
	0x80400000UL, 0x80001000UL, 0x00401040UL, 0x80400040UL,
	0x80001000UL, 0x00001040UL, 0x00400000UL, 0x80401000UL,
	0x00000040UL, 0x00400000UL, 0x00001000UL, 0x00401040UL,

	0x10000008UL, 0x10200000UL, 0x00002000UL, 0x10202008UL,	/* 6 */
	0x10200000UL, 0x00000008UL, 0x10202008UL, 0x00200000UL,
	0x10002000UL, 0x00202008UL, 0x00200000UL, 0x10000008UL,
	0x00200008UL, 0x10002000UL, 0x10000000UL, 0x00002008UL,
	0x00000000UL, 0x00200008UL, 0x10002008UL, 0x00002000UL,
	0x00202000UL, 0x10002008UL, 0x00000008UL, 0x10200008UL,
	0x10200008UL, 0x00000000UL, 0x00202008UL, 0x10202000UL,
	0x00002008UL, 0x00202000UL, 0x10202000UL, 0x10000000UL,
	0x10002000UL, 0x00000008UL, 0x10200008UL, 0x00202000UL,
	0x10202008UL, 0x00200000UL, 0x00002008UL, 0x10000008UL,
	0x00200000UL, 0x10002000UL, 0x10000000UL, 0x00002008UL,
	0x10000008UL, 0x10202008UL, 0x00202000UL, 0x10200000UL,
	0x00202008UL, 0x10202000UL, 0x00000000UL, 0x10200008UL,
	0x00000008UL, 0x00002000UL, 0x10200000UL, 0x00202008UL,
	0x00002000UL, 0x00200008UL, 0x10002008UL, 0x00000000UL,
	0x10202000UL, 0x10000000UL, 0x00200008UL, 0x10002008UL,

	0x08000820UL, 0x00000800UL, 0x00020000UL, 0x08020820UL,	/* 8 */
	0x08000000UL, 0x08000820UL, 0x00000020UL, 0x08000000UL,
	0x00020020UL, 0x08020000UL, 0x08020820UL, 0x00020800UL,
	0x08020800UL, 0x00020820UL, 0x00000800UL, 0x00000020UL,
	0x08020000UL, 0x08000020UL, 0x08000800UL, 0x00000820UL,
	0x00020800UL, 0x00020020UL, 0x08020020UL, 0x08020800UL,
	0x00000820UL, 0x00000000UL, 0x00000000UL, 0x08020020UL,
	0x08000020UL, 0x08000800UL, 0x00020820UL, 0x00020000UL,
	0x00020820UL, 0x00020000UL, 0x08020800UL, 0x00000800UL,
	0x00000020UL, 0x08020020UL, 0x00000800UL, 0x00020820UL,
	0x08000800UL, 0x00000020UL, 0x08000020UL, 0x08020000UL,
	0x08020020UL, 0x08000000UL, 0x00020000UL, 0x08000820UL,
	0x00000000UL, 0x08020820UL, 0x00020020UL, 0x08000020UL,
	0x08020000UL, 0x08000800UL, 0x08000820UL, 0x00000000UL,
	0x08020820UL, 0x00020800UL, 0x00020800UL, 0x00000820UL,
	0x00000820UL, 0x00020020UL, 0x08000000UL, 0x08020800UL,

	0x40084010UL, 0x40004000UL, 0x00004000UL, 0x00084010UL,	/* 2 */
	0x00080000UL, 0x00000010UL, 0x40080010UL, 0x40004010UL,
	0x40000010UL, 0x40084010UL, 0x40084000UL, 0x40000000UL,
	0x40004000UL, 0x00080000UL, 0x00000010UL, 0x40080010UL,
	0x00084000UL, 0x00080010UL, 0x40004010UL, 0x00000000UL,
	0x40000000UL, 0x00004000UL, 0x00084010UL, 0x40080000UL,
	0x00080010UL, 0x40000010UL, 0x00000000UL, 0x00084000UL,
	0x00004010UL, 0x40084000UL, 0x40080000UL, 0x00004010UL,
	0x00000000UL, 0x00084010UL, 0x40080010UL, 0x00080000UL,
	0x40004010UL, 0x40080000UL, 0x40084000UL, 0x00004000UL,
	0x40080000UL, 0x40004000UL, 0x00000010UL, 0x40084010UL,
	0x00084010UL, 0x00000010UL, 0x00004000UL, 0x40000000UL,
	0x00004010UL, 0x40084000UL, 0x00080000UL, 0x40000010UL,
	0x00080010UL, 0x40004010UL, 0x40000010UL, 0x00080010UL,
	0x00084000UL, 0x00000000UL, 0x40004000UL, 0x00004010UL,
	0x40000000UL, 0x40080010UL, 0x40084010UL, 0x00084000UL
};
