/*
 * @OSF_COPYRIGHT@
 * COPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993, 1996 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (DCE).  See the file named COPYRIGHT.DCE in the
 * src directory for the full copyright text.
 */
/*
 * HISTORY
 * $Log: weak_key.c,v $
 * Revision 1.1.8.2  1996/02/17  23:59:44  marty
 * 	Update OSF copyright years
 * 	[1996/02/17  22:53:17  marty]
 *
 * Revision 1.1.8.1  1995/12/08  00:13:35  root
 * 	Submit OSF/DCE 1.2.1
 * 	[1995/12/07  23:55:25  root]
 * 
 * Revision 1.1.6.1  1994/01/21  22:30:58  cbrooks
 * 	Code Cleanup
 * 	[1994/01/21  20:20:56  cbrooks]
 * 
 * Revision 1.1.4.3  1993/01/03  22:34:45  bbelch
 * 	Embedding copyright notice
 * 	[1993/01/03  19:49:56  bbelch]
 * 
 * Revision 1.1.4.2  1992/12/23  19:35:57  zeliff
 * 	Embedding copyright notice
 * 	[1992/12/23  01:15:36  zeliff]
 * 
 * Revision 1.1.2.2  1992/05/01  17:36:40  rsalz
 * 	"Changed pas part of rpc6 drop."
 * 	[1992/05/01  17:34:20  rsalz]
 * 
 * Revision 1.1  1992/01/19  03:17:00  devrcs
 * 	Initial revision
 * 
 * $EndLog$
 */
/*
**  Copyright (c) 1991 by
**      Hewlett-Packard Company, Palo Alto, Ca. & 
**
**
**  NAME
**
**      weak_key.c
**
**  FACILITY:
**
**      Remote Procedure Call (RPC) 
**
**  ABSTRACT:
**
**  Kernel implementation of weak DES key checks.
**
**
*/

/*
 * $Source: /u0/rcs_trees/dce/rcs/rpc/kdes/weak_key.c,v $
 * $Author: marty $
 *
 * Copyright 1989 by the Massachusetts Institute of Technology.
 *
 * For copying and distribution information, please see the file
 * <krb5/copyright.h>.
 *
 * Under U.S. law, this software may not be exported outside the US
 * without license from the U.S. Commerce department.
 *
 * These routines form the library interface to the DES facilities.
 *
 * Originally written 8/85 by Steve Miller, MIT Project Athena.
 */

#if !defined(lint) && !defined(SABER)
static char rcsid_weak_key_c[] =
"$Id: weak_key.c,v 1.1.8.2 1996/02/17 23:59:44 marty Exp $";
#endif  /* !lint & !SABER */

#include <dce/dce.h>
#include <des.h>

/*
 * The following are the weak DES keys:
 */
static des_cblock weak[16] = {
    /* weak keys */
    {0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01},
    {0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe},
    {0x1f,0x1f,0x1f,0x1f,0x0e,0x0e,0x0e,0x0e},
    {0xe0,0xe0,0xe0,0xe0,0xf1,0xf1,0xf1,0xf1},

    /* semi-weak */
    {0x01,0xfe,0x01,0xfe,0x01,0xfe,0x01,0xfe},
    {0xfe,0x01,0xfe,0x01,0xfe,0x01,0xfe,0x01},

    {0x1f,0xe0,0x1f,0xe0,0x0e,0xf1,0x0e,0xf1},
    {0xe0,0x1f,0xe0,0x1f,0xf1,0x0e,0xf1,0x0e},

    {0x01,0xe0,0x01,0xe0,0x01,0xf1,0x01,0xf1},
    {0xe0,0x01,0xe0,0x01,0xf1,0x01,0xf1,0x01},

    {0x1f,0xfe,0x1f,0xfe,0x0e,0xfe,0x0e,0xfe},
    {0xfe,0x1f,0xfe,0x1f,0xfe,0x0e,0xfe,0x0e},

    {0x01,0x1f,0x01,0x1f,0x01,0x0e,0x01,0x0e},
    {0x1f,0x01,0x1f,0x01,0x0e,0x01,0x0e,0x01},

    {0xe0,0xfe,0xe0,0xfe,0xf1,0xfe,0xf1,0xfe},
    {0xfe,0xe0,0xfe,0xe0,0xfe,0xf1,0xfe,0xf1}
};

/*
 * sec_des_is_weak_key: returns true iff key is a [semi-]weak des key.
 *
 * Requires: key has correct odd parity.
 */
int
sec_des_is_weak_key
#ifdef _DCE_PROTO 
     (
     des_cblock key;
      )
#else
      (key)
	des_cblock key ;
#endif /* _DCE_PROTO_ */
{
    int i;
    des_cblock *weak_p = weak;

    for (i = 0; i < (sizeof(weak)/sizeof(des_cblock)); i++) {
        if (!memcmp((char *)weak_p++,(char *)key,sizeof(des_cblock)))
            return 1;
    }

    return 0;
}
