/*
 * @OSF_COPYRIGHT@
 * COPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993, 1996 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (DCE).  See the file named COPYRIGHT.DCE in the
 * src directory for the full copyright text.
 */
/*
 * HISTORY
 * $Log: cksum.c,v $
 * Revision 1.1.8.2  1996/02/18  00:10:29  marty
 * 	Update OSF copyright years
 * 	[1996/02/17  23:01:44  marty]
 *
 * Revision 1.1.8.1  1995/12/08  17:49:24  root
 * 	Submit OSF/DCE 1.2.1
 * 
 * 	HP revision /main/HPDCE02/1  1995/08/21  17:46 UTC  sommerfeld
 * 	Use Outerbridge DES -- add temporary variable here.
 * 	[1995/08/01  19:26 UTC  sommerfeld  /main/sommerfeld_dfsperf/1]
 * 
 * Revision 1.1.6.1  1993/10/14  17:45:27  mccann
 * 	CR8651 64bit port
 * 	[1993/10/14  17:43:14  mccann]
 * 
 * Revision 1.1.4.2  1992/12/29  14:23:05  zeliff
 * 	Embedding copyright notice
 * 	[1992/12/28  21:04:58  zeliff]
 * 
 * Revision 1.1.2.2  1992/05/18  15:09:21  pato
 * 	Add mit_des_cbc_cksum
 * 	[1992/05/15  20:42:15  pato]
 * 
 * 	Initial revision
 * 	[1992/05/14  15:02:33  pato]
 * 
 * $EndLog$
 */

/*
** Copyright (c) Hewlett-Packard Company 1991, 1992
** Unpublished work. All Rights Reserved.
*/

/*
 * Copyright (c) 1990 Dennis Ferguson.  All rights reserved.
 *
 * Commercial use is permitted only if products which are derived from
 * or include this software are made available for purchase and/or use
 * in Canada.  Otherwise, redistribution and use in source and binary
 * forms are permitted.
 */

#include <krb5/krb5.h>
#include <krb5/mit-des.h>
#include <krb5/des_int.h>

/*
 * des_cbc_cksum.c - compute an 8 byte checksum using DES in CBC mode
 */
#include "des_tables.h"


krb5_ui_4
des_cbc_cksum(  DECLARG(krb5_octet *, in),
                    DECLARG(krb5_octet *, out),
                    DECLARG(krb5_ui_4, length),
                    DECLARG(mit_des_key_schedule, schedule),
                    DECLARG(krb5_octet *, ivec))
OLDDECLARG(krb5_octet *, in)
OLDDECLARG(krb5_octet *, out)
OLDDECLARG(krb5_ui_4, length)
OLDDECLARG(mit_des_key_schedule, schedule)
OLDDECLARG(krb5_octet *, ivec)
{
	register krb5_ui_4 left, right;
	register krb5_ui_4 temp, temp1;
	register krb5_ui_4 *kp;
	register unsigned char *ip;
	register krb5_ui_4 len;

	/*
	 * Initialize left and right with the contents of the initial
	 * vector.
	 */
	ip = (unsigned char *)ivec;
	GET_HALF_BLOCK(left, ip);
	GET_HALF_BLOCK(right, ip);

	/*
	 * Suitably initialized, now work the length down 8 bytes
	 * at a time.
	 */
	ip = (unsigned char *)in;
	len = length;
	while (len > 0) {
		/*
		 * Get more input, xor it in.  If the length is
		 * greater than or equal to 8 this is straight
		 * forward.  Otherwise we have to fart around.
		 */
		if (len >= 8) {
			left ^= ((*ip++) & 0xff) << 24;
			left ^= ((*ip++) & 0xff) << 16;
			left ^= ((*ip++) & 0xff) << 8;
			left ^= (*ip++) & 0xff;
			right ^= ((*ip++) & 0xff) << 24;
			right ^= ((*ip++) & 0xff) << 16;
			right ^= ((*ip++) & 0xff) << 8;
			right ^= (*ip++) & 0xff;
			len -= 8;
		} else {
			/*
			 * Oh, shoot.  We need to pad the
			 * end with zeroes.  Work backwards
			 * to do this.
			 */
			ip += (int) len;
			switch(len) {
			case 7:
				right ^= (*(--ip) & 0xff) << 8;
			case 6:
				right ^= (*(--ip) & 0xff) << 16;
			case 5:
				right ^= (*(--ip) & 0xff) << 24;
			case 4:
				left ^= *(--ip) & 0xff;
			case 3:
				left ^= (*(--ip) & 0xff) << 8;
			case 2:
				left ^= (*(--ip) & 0xff) << 16;
			case 1:
				left ^= (*(--ip) & 0xff) << 24;
				break;
			}
			len = 0;
		}

		/*
		 * Encrypt what we have
		 */
		kp = (krb5_ui_4 *)schedule;
		DES_DO_ENCRYPT(left, right, temp, temp1, kp);
	}

	/*
	 * Done.  Left and right have the checksum.  Put it into
	 * the output.
	 */
	ip = (unsigned char *)out;
	PUT_HALF_BLOCK(left, ip);
	PUT_HALF_BLOCK(right, ip);

	/*
	 * Return right.  I'll bet the MIT code returns this
	 * inconsistantly (with the low order byte of the checksum
	 * not always in the low order byte of the long).  We won't.
	 */
	return right;
}

void mit_des_cbc_cksum(  
                    DECLARG(krb5_octet *, in),
                    DECLARG(krb5_octet *, out),
                    DECLARG(krb5_int32, length),
                    DECLARG(mit_des_key_schedule, key),
                    DECLARG(krb5_octet *, iv))
OLDDECLARG(krb5_octet *, in)            /* >= length bytes of inputtext */
OLDDECLARG(krb5_octet *, out)           /* >= length bytes of outputtext */
OLDDECLARG(krb5_int32, length)                /* in bytes */
OLDDECLARG(mit_des_key_schedule, key)   /* precomputed key schedule */
OLDDECLARG(krb5_octet *, iv)            /* 8 bytes of ivec */
{
    (void) des_cbc_cksum(in, out, length, key, iv);
}

