/*
 * @OSF_COPYRIGHT@
 * (left)TEMP)OPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993, 1996 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (DCE).  See the file named COPYRIGHT.DCE in the
 * src directory for the full copyright text.
 */
/*
 * HISTORY
 * $Log: des_tables.h,v $
 * Revision 1.1.8.2  1996/02/18  23:03:24  marty
 * 	Update OSF copyright years
 * 	[1996/02/18  22:21:56  marty]
 *
 * Revision 1.1.8.1  1995/12/08  17:49:33  root
 * 	Submit OSF/DCE 1.2.1
 * 
 * 	HP revision /main/HPDCE02/2  1995/08/25  23:04 UTC  sommerfeld
 * 	CHFts16189: de-const the tables due to linker bug affecting intl build.
 * 
 * 	HP revision /main/HPDCE02/1  1995/08/21  17:46 UTC  sommerfeld
 * 	Change name of rotate macro.
 * 	Namespace-safe the table names.
 * 	Remove remnants of (now useless) Ferguson DES tables.
 * 	[1995/08/02  15:41 UTC  sommerfeld  /main/sommerfeld_dfsperf/2]
 * 
 * 	use Richard Outerbridge DES..
 * 	[1995/08/01  19:28 UTC  sommerfeld  /main/sommerfeld_dfsperf/1]
 * 
 * Revision 1.1.6.1  1993/10/14  17:45:29  mccann
 * 	CR8651 64bit port
 * 	[1993/10/14  17:43:38  mccann]
 * 
 * Revision 1.1.4.2  1992/12/29  14:23:29  zeliff
 * 	Embedding copyright notice
 * 	[1992/12/28  21:05:08  zeliff]
 * 
 * Revision 1.1.2.2  1992/05/18  15:10:27  pato
 * 	Initial revision
 * 	[1992/05/14  15:06:29  pato]
 * 
 * $EndLog$
 */

/*
** Copyright (c) Hewlett-Packard Company 1991, 1992
** Unpublished work. All Rights Reserved.
*/

/*
 * Copyright (c) 1990 Dennis Ferguson.  All rights reserved.
 *
 * Commercial use is permitted only if products which are derived from
 * or include this software are made available for purchase and/or use
 * in Canada.  Otherwise, redistribution and use in source and binary
 * forms are permitted.
 */

/*
 * des_tables.h - declarations to import the DES tables, used internally
 *		  by some of the library routines.
 */
#ifndef	__DES_TABLES_H__
#define	__DES_TABLES_H__	/* nothing */


#define SP1 des_SP_table1
#define SP2 des_SP_table2
#define SP3 des_SP_table3
#define SP4 des_SP_table4
#define SP5 des_SP_table5
#define SP6 des_SP_table6
#define SP7 des_SP_table7
#define SP8 des_SP_table8

extern krb5_ui_4 SP1[64], SP2[64], SP3[64], SP4[64], SP5[64], SP6[64], SP7[64], SP8[64]; 
/*
 * Use standard shortforms to reference these to save typing
 */

/* rotate right by N bits */
#define U32ROTR(value,shift) (((value)<<(32-(shift)))|((value)>>(shift)))
#define U32ROTL(value,shift) (((value)>>(32-(shift)))|((value)<<(shift)))

/*
 * Code to do a DES round using the tables.  Note that the E expansion
 * is easy to compute algorithmically, especially if done out-of-order.
 * Take a look at its form and compare it to everything involving temp
 * below.  Since SP[0-7] don't have any bits in common set it is okay
 * to do the successive xor's.
 *
 * Note too that the SP table has been reordered to match the order of
 * the keys (if the original order of SP was 12345678, the reordered
 * table is 71354682).  This is unnecessary, but was done since some
 * compilers seem to like you going through the matrix from beginning
 * to end.
 *
 * There is a difference in the best way to do this depending on whether
 * one is encrypting or decrypting.  If encrypting we move forward through
 * the keys and hence should move forward through the table.  If decrypting
 * we go back.  Part of the need for this comes from trying to emulate
 * existing software which generates a single key schedule and uses it
 * both for encrypting and decrypting.  Generating separate encryption
 * and decryption key schedules would allow one to use the same code
 * for both.
 *
 * left, right and temp should be unsigned long values.  left and right
 * should be the high and low order parts of the cipher block at the
 * current stage of processing (this makes sense if you read the spec).
 * kp should be an unsigned long pointer which points at the current
 * set of subkeys in the key schedule.  It is advanced to the next set
 * (i.e. by 8 bytes) when this is done.
 *
 * This occurs in the innermost loop of the DES function.  The four
 * variables should really be in registers.
 *
 * When using this, the inner loop of the DES function might look like:
 *
 *	for (i = 0; i < 8; i++) {
 *		DES_SP_{EN,DE}CRYPT_ROUND(left, right, temp, temp1, kp);
 *		DES_SP_{EN,DE}CRYPT_ROUND(right, left, temp, temp1, kp);
 *	}
 *
 * Note the trick above.  You are supposed to do 16 rounds, swapping
 * left and right at the end of each round.  By doing two rounds at
 * a time and swapping left and right in the code we can avoid the
 * swaps altogether.
 */

#if 0
#define	DES_SP_ENCRYPT_ROUND(left, right, temp, temp1, kp) \
	(temp) = (((right) >> 11) | ((right) << 21)) ^ *(kp)++; \
	(left) ^= SP[0][((temp) >> 24) & 0x3f] \
		| SP[1][((temp) >> 16) & 0x3f] \
		| SP[2][((temp) >>  8) & 0x3f] \
		| SP[3][((temp)      ) & 0x3f]; \
	(temp) = (((right) >> 23) | ((right) << 9)) ^ *(kp)++; \
	(left) ^= SP[4][((temp) >> 24) & 0x3f] \
		| SP[5][((temp) >> 16) & 0x3f] \
		| SP[6][((temp) >>  8) & 0x3f] \
		| SP[7][((temp)      ) & 0x3f]

#define	DES_SP_DECRYPT_ROUND(left, right, temp, temp1, kp) \
	(temp) = (((right) >> 23) | ((right) << 9)) ^ *(--(kp)); \
	(left) ^= SP[7][((temp)      ) & 0x3f] \
		| SP[6][((temp) >>  8) & 0x3f] \
		| SP[5][((temp) >> 16) & 0x3f] \
		| SP[4][((temp) >> 24) & 0x3f]; \
	(temp) = (((right) >> 11) | ((right) << 21)) ^ *(--(kp)); \
	(left) ^= SP[3][((temp)      ) & 0x3f] \
		| SP[2][((temp) >>  8) & 0x3f] \
		| SP[1][((temp) >> 16) & 0x3f] \
		| SP[0][((temp) >> 24) & 0x3f]
#endif

/*
 * these would be faster, but unfortunately they require a change
 * to the key schedule generation code ...
 */
#define	DES_SP_ENCRYPT_ROUND(left, right, temp, temp1, kp) \
		(temp)  = U32ROTR((right), 4);		\
		(temp) ^= *(kp)++; \
		(temp1)  = SP7[ (temp)        & 0x3fL]; \
		(temp1) |= SP5[((temp) >>  8) & 0x3fL]; \
		(temp1) |= SP3[((temp) >> 16) & 0x3fL]; \
		(temp1) |= SP1[((temp) >> 24) & 0x3fL]; \
		(temp)  = (right) ^ *(kp)++; \
		(temp1) |= SP8[ (temp)        & 0x3fL]; \
		(temp1) |= SP6[((temp) >>  8) & 0x3fL]; \
		(temp1) |= SP4[((temp) >> 16) & 0x3fL]; \
		(temp1) |= SP2[((temp) >> 24) & 0x3fL]; \
		(left) ^= (temp1);

#define	DES_SP_DECRYPT_ROUND(left, right, temp, temp1, kp) \
		(temp)  = (right) ^ *(--(kp)); \
		(temp1)  = SP8[ (temp)        & 0x3fL]; \
		(temp1) |= SP6[((temp) >>  8) & 0x3fL]; \
		(temp1) |= SP4[((temp) >> 16) & 0x3fL]; \
		(temp1) |= SP2[((temp) >> 24) & 0x3fL]; \
		(temp)  = U32ROTR((right), 4);		\
		(temp) ^= *(--(kp)); \
		(temp1) |= SP7[ (temp)        & 0x3fL]; \
		(temp1) |= SP5[((temp) >>  8) & 0x3fL]; \
		(temp1) |= SP3[((temp) >> 16) & 0x3fL]; \
		(temp1) |= SP1[((temp) >> 24) & 0x3fL]; \
		(left) ^= (temp1);

/*
 * The following macro does an in-place initial permutation given
 * the current left and right parts of the block and a single
 * temporary.  Use this more as a guide for rolling your own, though.
 * The best way to do the IP depends on the form of the data you
 * are dealing with.  If you use this, though, try to make left,
 * right and temp register unsigned longs.
 */
#define	DES_INITIAL_PERM(left, right, temp) \
	(temp) = (((left) >> 4) ^ (right)) & 0x0f0f0f0fL; \
	(right) ^= (temp); \
	(left) ^= ((temp) << 4); \
	(temp) = (((left) >> 16) ^ (right)) & 0x0000ffffL; \
	(right) ^= (temp); \
	(left) ^= ((temp) << 16); \
	(temp) = (((right) >> 2) ^ (left)) & 0x33333333L; \
	(left) ^= (temp); \
	(right) ^= ((temp) << 2); \
	(temp) = (((right) >> 8) ^ (left)) & 0x00ff00ffL; \
	(left) ^= (temp); \
	(right) ^= ((temp) << 8); \
	(right) = U32ROTL((right),1) & 0xffffffffL; \
	(temp) = ((left) ^ (right)) & 0xaaaaaaaaL; \
	(left) ^= (temp); \
	(right) ^= (temp); \
	(left) = U32ROTL((left),1) & 0xffffffffL;


/*
 * Here is a sample final permutation.  DES requires swapping the left
 * and right parts after the last cipher round but before the final
 * permutation.  We do this swapping internally, which is why left and
 * right are confused at the beginning.
 */


#define DES_FINAL_PERM(left, right, temp) \
	(right) = U32ROTR((right),1); \
	(temp) = ((left) ^ (right)) & 0xaaaaaaaaL; \
	(left) = (temp) ^ (left); \
	(right) = (temp) ^ (right); \
	(left) = U32ROTR((left),1); \
	(temp) = (((left) >> 8) ^ (right)) & 0x00ff00ffL; \
	(right) ^= (temp); \
	(left) ^= ((temp) << 8); \
	(temp) = ((left >> 2) ^ (right)) & 0x33333333L; \
	(right) ^= (temp); \
	left ^= ((temp) << 2); \
	(temp) = (((right) >> 16) ^ left) & 0x0000ffffL; \
	left ^= temp; \
	(right) ^= (temp << 16); \
	(temp) = (((right) >> 4) ^ left) & 0x0f0f0f0fL; \
	temp1 = (left) ^ (temp); \
	(left) = (right) ^ (temp << 4); \
	(right) = temp1;	/* !!! can we do swap better? */


/*
 * Finally, as a sample of how all this might be held together, the
 * following two macros do in-place encryptions and decryptions.  left
 * and right are two unsigned long variables which at the beginning
 * are expected to hold the clear (encrypted) block in host byte order
 * (left the high order four bytes, right the low order).  At the end
 * they will contain the encrypted (clear) block.  temp is an unsigned long
 * used as a temporary.  kp is an unsigned long pointer pointing at
 * the start of the key schedule.  All these should be in registers.
 *
 * You can probably do better than these by rewriting for particular
 * situations.  These aren't bad, though.
 */
#define	DES_DO_ENCRYPT(left, right, temp, temp1, kp) \
	do { \
		register int i; \
		DES_INITIAL_PERM((left), (right), (temp)); \
		for (i = 0; i < 8; i++) { \
			DES_SP_ENCRYPT_ROUND((left), (right), (temp), (temp1), (kp)); \
			DES_SP_ENCRYPT_ROUND((right), (left), (temp), (temp1), (kp)); \
		} \
		DES_FINAL_PERM((left), (right), (temp)); \
		(kp) -= (2 * 16); \
	} while (0)

#define	DES_DO_DECRYPT(left, right, temp, temp1, kp) \
	do { \
		register int i; \
		DES_INITIAL_PERM((left), (right), (temp)); \
		(kp) += (2 * 16); \
		for (i = 0; i < 8; i++) { \
			DES_SP_DECRYPT_ROUND((left), (right), (temp), (temp1), (kp)); \
			DES_SP_DECRYPT_ROUND((right), (left), (temp), (temp1), (kp)); \
		} \
		DES_FINAL_PERM((left), (right), (temp)); \
	} while (0)

/*
 * These are handy dandy utility thingies for straightening out bytes.
 * Included here because they're used a couple of places.
 */
#define	GET_HALF_BLOCK(lr, ip) \
	(lr) = ((krb5_ui_4)(*(ip)++)) << 24; \
	(lr) |= ((krb5_ui_4)(*(ip)++)) << 16; \
	(lr) |= ((krb5_ui_4)(*(ip)++)) << 8; \
	(lr) |= (krb5_ui_4)(*(ip)++)

#define	PUT_HALF_BLOCK(lr, op) \
	*(op)++ = ((lr) >> 24) & 0xff; \
	*(op)++ = ((lr) >> 16) & 0xff; \
	*(op)++ = ((lr) >> 8) & 0xff; \
	*(op)++ = (lr) & 0xff

#endif	/* __DES_TABLES_H__ */
