/*
 * @OSF_COPYRIGHT@
 * COPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993, 1996 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (DCE).  See the file named COPYRIGHT.DCE in the
 * src directory for the full copyright text.
 */
/*
 * HISTORY
 * $Log: ecb_encrypt.c,v $
 * Revision 1.1.8.2  1996/02/18  00:10:37  marty
 * 	Update OSF copyright years
 * 	[1996/02/17  23:01:49  marty]
 *
 * Revision 1.1.8.1  1995/12/08  17:49:42  root
 * 	Submit OSF/DCE 1.2.1
 * 
 * 	HP revision /main/HPDCE02/2  1995/08/22  22:00 UTC  sommerfeld
 * 	take out what-string.
 * 	[1995/08/22  20:34 UTC  sommerfeld  /main/sommerfeld_mothpatch/1]
 * 
 * 	HP revision /main/HPDCE02/1  1995/08/21  17:46 UTC  sommerfeld
 * 	Use Outerbridge DES -- add temporary variable here.
 * 	[1995/08/01  19:23 UTC  sommerfeld  /main/sommerfeld_dfsperf/1]
 * 
 * Revision 1.1.6.1  1993/10/14  17:45:32  mccann
 * 	CR8651 64bit port
 * 	[1993/10/14  17:43:47  mccann]
 * 
 * Revision 1.1.4.2  1992/12/29  14:24:08  zeliff
 * 	Embedding copyright notice
 * 	[1992/12/28  21:05:17  zeliff]
 * 
 * Revision 1.1.2.2  1992/05/18  15:11:16  pato
 * 	Initial revision
 * 	[1992/05/14  15:07:34  pato]
 * 
 * $EndLog$
 */

/*
** Copyright (c) Hewlett-Packard Company 1991, 1992
** Unpublished work. All Rights Reserved.
*/

/*
 * Copyright (c) 1990 Dennis Ferguson.  All rights reserved.
 *
 * Commercial use is permitted only if products which are derived from
 * or include this software are made available for purchase and/or use
 * in Canada.  Otherwise, redistribution and use in source and binary
 * forms are permitted.
 */

#include <krb5/krb5.h>
#include <krb5/mit-des.h>
#include <krb5/des_int.h>

/*
 * des_ecb_encrypt.c - do an encryption in ECB mode
 */
#include "des_tables.h"

/*
 * des_ecb_encrypt - {en,de}crypt a block in ECB mode
 */
int
mit_des_ecb_encrypt(DECLARG(krb5_octet *, in),
                        DECLARG(krb5_octet *, out),
                        DECLARG(mit_des_key_schedule, schedule),
                        DECLARG(int, encrypt))
OLDDECLARG(krb5_octet *, in)
OLDDECLARG(krb5_octet *, out)
OLDDECLARG(mit_des_key_schedule, schedule)
OLDDECLARG(int, encrypt)
{
	register krb5_ui_4 left, right;
	register krb5_ui_4 temp, temp1;
	register int i;

	{
		/*
		 * Need a temporary for copying the data in
		 */
		register unsigned char *datap;

		/*
		 * Copy the input block into the registers
		 */
		datap = (unsigned char *)in;
		GET_HALF_BLOCK(left, datap);
		GET_HALF_BLOCK(right, datap);
	}

	/*
	 * Do the initial permutation.
	 */
	DES_INITIAL_PERM(left, right, temp);

	/*
	 * Now the rounds.  Use different code depending on whether it
	 * is an encryption or a decryption (gross, should keep both
	 * sets of keys in the key schedule instead).
	 */
	if (encrypt) {
		register krb5_ui_4 *kp;

		kp = (krb5_ui_4 *)schedule;
		for (i = 0; i < 8; i++) {
			DES_SP_ENCRYPT_ROUND(left, right, temp, temp1, kp);
			DES_SP_ENCRYPT_ROUND(right, left, temp, temp1, kp);
		}
	} else {
		register krb5_ui_4 *kp;

		/*
		 * Point kp past end of schedule
		 */
		kp = ((krb5_ui_4 *)schedule) + (2 * 16);;
		for (i = 0; i < 8; i++) {
			DES_SP_DECRYPT_ROUND(left, right, temp, temp1, kp);
			DES_SP_DECRYPT_ROUND(right, left, temp, temp1, kp);
		}
	}

	/*
	 * Do the final permutation
	 */
	DES_FINAL_PERM(left, right, temp);

	/*
	 * Finally, copy the result out a byte at a time
	 */
	{
		register unsigned char *datap;

		datap = (unsigned char *)out;
		PUT_HALF_BLOCK(left, datap);
		PUT_HALF_BLOCK(right, datap);
	}

	/*
	 * return nothing
	 */
	return (0);
}
